﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;
using System.Drawing;
using System.Collections;
using System.Drawing.Imaging;

namespace acmesoft.web.ascx
{
    public partial class BarChart : System.Web.UI.UserControl
    {
        public string ChartTitle = "";
        public int ImageHeight = 375;
        public int ImageWidth = 500;
        public string ImageAlt = "BarChartImage";

        public int PanelPadding = 17;
        public int PanelShadowDistance = 3;
        public int PanelShadowAlpha = 0;

        public int BarPaddingTop = 15;
        public int BarPaddingBottom = 15;
        public int BarPaddingLeft = 150;
        public int BarPaddingRight = 15;
        public int BarHeight = 1;

        public Point LabelPosition = new Point(30, 30);

        public Brush BackgroundBrush = null;
        public Brush TitleBrush = Brushes.White;
        public Brush PanelBackgroundBrush = Brushes.White;

        public class BarChartElement
        {
            public string Name;
            public float Percent;
            public Color Color;

            public BarChartElement(string name)
            {
                this.Name = name;
            }
            public BarChartElement(string name, float percent)
            {
                this.Name = name;
                this.Percent = percent;
            }
            public BarChartElement(string name, float percent, Color color)
            {
                this.Name = name;
                this.Percent = percent;
                this.Color = color;
            }
        }

        private ArrayList BarChartElements;

        protected void Page_Load(object sender, EventArgs e)
        {
            deleteOldPlotImages();
        }

        private void deleteOldPlotImages()
        {
            DirectoryInfo myDirectoryInfo = new DirectoryInfo(HttpContext.Current.Server.MapPath("~/elements/chartImages"));
            foreach (FileInfo myFileInfo in myDirectoryInfo.GetFiles())
            {
                if (myFileInfo.LastWriteTime.AddMinutes(10) < DateTime.Now)
                {
                    myFileInfo.Delete();
                }
            }
        }

        private void drawPanel(Graphics g)
        {
            // draw background
            Rectangle BackgroundRectangle = new Rectangle(0, 0, ImageWidth, ImageHeight);

            if (BackgroundBrush == null)
            {
                BackgroundBrush = new System.Drawing.Drawing2D.LinearGradientBrush(BackgroundRectangle, Color.FromArgb(156, 197, 250), Color.FromArgb(10, 64, 135), 0, false);
            }

            g.FillRectangle(BackgroundBrush, BackgroundRectangle);

            // draw title
            StringFormat titleStringFormat = new StringFormat();

            titleStringFormat.Alignment = StringAlignment.Center;

            Font titleFont = new Font("Tahoma", 10, FontStyle.Bold);

            g.DrawString(ChartTitle, titleFont, TitleBrush, new RectangleF(0, 0, ImageWidth, 20), titleStringFormat);

            // draw main panel
            Rectangle panelRectangle = new Rectangle(PanelPadding, PanelPadding, ImageWidth - 2 * PanelPadding, ImageHeight - 2 * PanelPadding);
            Rectangle panelShadowRectangle = new Rectangle(PanelPadding + PanelShadowDistance, PanelPadding + PanelShadowDistance, (ImageWidth - 2 * PanelPadding) + PanelShadowDistance, (ImageHeight - 2 * PanelPadding) + PanelShadowDistance);
            SolidBrush transparentSolidBrush = new SolidBrush(Color.FromArgb(PanelShadowAlpha, 0, 0, 0));

            g.FillRectangle(transparentSolidBrush, panelShadowRectangle);
            g.FillRectangle(PanelBackgroundBrush, panelRectangle);
            g.DrawRectangle(Pens.Black, panelRectangle);
        }

        private void drawLabels(Graphics g)
        {
            float x = (float)LabelPosition.X;
            float y = (float)LabelPosition.Y;

            Font labelFont = new Font("Tahoma", 9, FontStyle.Regular);

            foreach (BarChartElement myBarChartElement in BarChartElements)
            {
                Brush labelBrush = new SolidBrush(myBarChartElement.Color);
                g.FillRectangle(labelBrush, x, (float)(y + labelFont.Height / 2 - 5), 10.0f, 10.0f);                

                string labelText = myBarChartElement.Name + " " + (myBarChartElement.Percent + 0.001).ToString().Substring(0, 4) + "%";
                g.DrawString(labelText, labelFont, Brushes.Black, x + 11, y);
                y += labelFont.Height + 5;
            }
        }

        public void addBarChartElement(BarChartElement newBarChartElement)
        {
            if (this.BarChartElements == null) this.BarChartElements = new ArrayList();
            this.BarChartElements.Add(newBarChartElement);
        }

        public void generateChartImage()
        {
            Bitmap myBitmap = new Bitmap(ImageWidth, ImageHeight, PixelFormat.Format32bppArgb);

            long ticks = (long)(DateTime.Now.Ticks / 100000000);
            string filename = "~/elements/chartImages/" + ID + "_" + ticks.ToString() + ".png";

            BarChartImage.Width = ImageWidth;
            BarChartImage.Height = ImageHeight;
            BarChartImage.ImageUrl = filename;
            BarChartImage.AlternateText = ImageAlt;

            //FileInfo myFileInfo = new FileInfo(Server.MapPath(filename));
            //if (myFileInfo.Exists) return;

            Graphics myGraphics = Graphics.FromImage(myBitmap);

            drawPanel(myGraphics);
            drawBar(myGraphics);
            drawLabels(myGraphics);

            myBitmap.Save(Server.MapPath(filename), ImageFormat.Png);
        }

        private void drawBar(Graphics g)
        {
            float x = 170, y = 35;
            float maxPercent = 0;
            
            foreach (BarChartElement myBarChartElement in BarChartElements)
            {
                if (myBarChartElement.Percent > maxPercent)
                    maxPercent = myBarChartElement.Percent;
            }
            float maxHeight = ImageHeight - 70;
            float scale = maxHeight / maxPercent;
            int itemCount = BarChartElements.Count;
            float width = 300.0f / itemCount;
            foreach (BarChartElement myBarChartElement in BarChartElements)
            {
                Brush labelBrush = new SolidBrush(myBarChartElement.Color);
                g.FillRectangle(labelBrush, x, y + (maxHeight - myBarChartElement.Percent * scale) , width, (myBarChartElement.Percent * scale));
                x += width + 1;
            }
        }
    }
}